﻿const mfs500Client = (function () {
    let isGetSuccess = false;

    function getHttpError(error) {
        if (error instanceof Response) {
            if (error.status === 0) {
                return 'Service Unavailable';
            } else if (error.status === 404) {
                return 'Requested page not found';
            } else if (error.status === 500) {
                return 'Internal Server Error';
            } else {
                return 'Unhandled Error';
            }
        } else {
            return "Network Error or Unhandled Exception: " + error.message;
        }
    }

    async function prepareScanner(ipAddress) {
        try {
            if (!isGetSuccess) {
                var res = await getMFScanInfo(ipAddress, 'MFS500')
                if (!res.httpStatus) {
                    return false;
                }
                isGetSuccess = true;
            }

        } catch (e) {
            console.error("prepareMFS500Client Error: ", e);
            return false;
        }

        return true;
    }

    function getFalseRes() {
        return {httpStatus: false, error: "Error while calling service"};
    }

    async function postMFScanClient(ipAddress, method, jsonData, isBodyAvailable) {
        try {
            let baseUrl = "http://" + ipAddress + ":8034/mfscan/";
            const response = await fetch(baseUrl + method, {
                method: "POST",
                mode: "cors",
                headers: {
                    "Content-Type": "application/json; charset=utf-8",
                },
                body: jsonData,
            });

            if (response.ok) {
                const data = await response.json();
                return {httpStatus: true, data: data};
            } else {
                return {httpStatus: false, error: getHttpError(response)};
            }
        } catch (error) {
            return {httpStatus: false, error: getHttpError(error)};
        }
    }

    async function getMFScanClient(ipAddress, method) {
        try {
            let baseUrl = "http://" + ipAddress + ":8034/mfscan/";
            const response = await fetch(baseUrl + method, {
                method: "GET",
                headers: {
                    "Content-Type": "application/json; charset=utf-8",
                },
            });

            if (response.ok) {
                const data = await response.json();
                return {httpStatus: true, data: data};
            } else {
                return {httpStatus: false, error: getHttpError(response)};
            }
        } catch (error) {
            return {httpStatus: false, error: getHttpError(error)};
        }

    }


    async function getMFScanInfo(ipAddress, connectedDvc, clientKey) {
        var MFScanRequest = {
            "ConnectedDvc": connectedDvc,
            "ClientKey": clientKey
        };

        var jsondata = JSON.stringify(MFScanRequest);
        return postMFScanClient(ipAddress, "info", jsondata);
    }

    async function captureFinger(ipAddress, scannerId, quality, timeout) {
        if (!prepareScanner('127.0.0.1')) {
            var errorFingerprint = {
                "successful": false,
                "scannerId": scannerId,
                "remarks": "Couldn't contact device"
            }

            await inayaFingerprintDevice.postScannedImageToInaya(errorFingerprint);
            return
        }

        const mfScanRequest = {
            "Quality": quality,
            "TimeOut": timeout
        };

        const jsondata = JSON.stringify(mfScanRequest);
        var result = await postMFScanClient(ipAddress, "capture", jsondata);

        var fingerprint;
        if (result.httpStatus) {
            if (result.data.ErrorCode == 0) {
                fingerprint = {
                    "successful": true,
                    "scannerId": scannerId,
                    "image": result.data.BitmapData,
                    "quality": result.data.Quality,
                    "template": result.data.IsoTemplate,
                }
            } else {
                fingerprint = {
                    "successful": false,
                    "scannerId": scannerId,
                    "remarks": result.data.ErrorDescription
                }
            }
        } else {
            fingerprint = {
                "successful": false,
                "scannerId": scannerId,
                "remarks": result.error
            }
        }

        await inayaFingerprintDevice.postScannedImageToInaya(fingerprint);
    }


    async function init() {
        var res = await getMFScanInfo('127.0.0.1', 'MFS500');
        if (res.httpStatus) {
            if (res.data.ErrorCode == "0") {
                inayaFingerprintDevice.registerDeviceOnInaya(
                    'MFS500',
                    res.data.DeviceInfo.LocalIP,
                    res.data.DeviceInfo.Make + "-" + res.data.DeviceInfo.Model + "-" + res.data.DeviceInfo.SerialNo);
            }
        } else {
            console.log("failed to init fingerprint device: " + res.error);
        }
    }


    function initDevice(ipAddress, connectedDvc, clientKey) {
        var MFScanRequest = {
            "ConnectedDvc": connectedDvc,
            "ClientKey": clientKey
        };
        var jsondata = JSON.stringify(MFScanRequest);
        return postMFScanClient(ipAddress, "initdevice", jsondata);
    }

    function uninitDevice(ipAddress,) {
        return postMFScanClient(ipAddress, "uninitdevice", "", 0);
    }

    function isDeviceConnected(ipAddress, connectedDvc) {
        var MFScanRequest = {
            "ConnectedDvc": connectedDvc
        };
        var jsondata = JSON.stringify(MFScanRequest);
        return postMFScanClient(ipAddress, "checkdevice", jsondata);
    }

    function getSupportedDeviceList() {
        return postMFScanClient("supporteddevicelist", "", 0);
    }

    function getConnectedDeviceList() {
        return postMFScanClient("connecteddevicelist", "", 0);
    }


    function getMFScanKeyInfo(key) {
        var MFScanRequest = {
            "Key": key,
        };
        var jsondata = JSON.stringify(MFScanRequest);
        return postMFScanClient("keyinfo", jsondata);
    }


    function getImage(imgformat) {
        var MFScanRequest = {
            "ImgFormat": imgformat
        };
        var jsondata = JSON.stringify(MFScanRequest);
        return postMFScanClient("getimage", jsondata);
    }

    init();

    return {
        captureFinger: captureFinger,

        getMFScanInfo: getMFScanInfo,
        initDevice: initDevice,
        uninitDevice: uninitDevice,
        isDeviceConnected: isDeviceConnected,
        getSupportedDeviceList: getSupportedDeviceList,
        getConnectedDeviceList: getConnectedDeviceList,
        getMFScanKeyInfo: getMFScanKeyInfo,
        getImage: getImage,
    }

})();

